/*
 * Event log factory example.
 */

#include "EventLogExample.h"


#define LOG_EVENTS 10

EventLogExample::EventLogExample (CORBA::ORB_var orbVar, ObjectAdapter *pAdapter)
 : POA_CosEventComm::PushSupplier (), orbVar_ (orbVar), pAdapter_ (pAdapter), eventLogFactoryVar_ (0), proxyVar_ (0)
{
  if (CORBA::is_nil (orbVar_.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: ORB not initialised\n")));
      exit (-1);
    }

  // Resolve to the event log factory.
  CORBA::Object_var objVar = 0;
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Resolving to event log factory\n")));
  ACE_TRY_NEW_ENV
    {
      objVar = orbVar_->resolve_initial_references (ACE_TEXT ("EventLogFactory") ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to resolve to event log factory\n"));
      exit (-1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to resolve to event log factory\n")));
      exit (-1);
    }
  ACE_ENDTRY;

  eventLogFactoryVar_ = DsEventLogAdmin::EventLogFactory::_narrow (objVar.in ());
  if (CORBA::is_nil (eventLogFactoryVar_.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Narrowed to a null event log factory\n")));
      exit (-1);
    }
}

EventLogExample::~EventLogExample ()
{
  // Workaround for the event log factory having a pure virtual destructor.
  (void) eventLogFactoryVar_._retn ();
}

DsEventLogAdmin::EventLog_ptr
EventLogExample::create ()
{
  DsEventLogAdmin::EventLog_var eventLogVar;
  DsLogAdmin::LogFullActionType logFullAction = DsLogAdmin::wrap;
  CORBA::ULongLong max_size = 0; // 0 means "infinite"
  DsLogAdmin::CapacityAlarmThresholdList_var thresVar = new DsLogAdmin::CapacityAlarmThresholdList ((CORBA::ULong)75);
  DsLogAdmin::LogId logId = 0;

  ACE_TRY_NEW_ENV
    {
      eventLogVar = eventLogFactoryVar_->create (logFullAction, max_size, thresVar.in (), logId ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCH (DsLogAdmin::InvalidLogFullAction, ilfa)
    {
      ACE_PRINT_EXCEPTION (ilfa, ACE_TEXT ("ERROR: Event log creation failed with InvalidLogFullAction\n"));
      exit (1);
    }
  ACE_CATCH (DsLogAdmin::InvalidThreshold, it)
    {
      ACE_PRINT_EXCEPTION (it, ACE_TEXT ("ERROR: Event log creation failed with InvalidThreshold\n"));
      exit (1);
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Event log creation failed\n"));
      exit (1);
    }
  ACE_ENDTRY;

  if (CORBA::is_nil (eventLogVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Event log is null\n")));
      exit (1);
    }

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Created log with id: %i\n"), logId));
  return eventLogVar._retn ();
}

void
EventLogExample::connect (DsEventLogAdmin::EventLog_ptr channel)
{
  CORBA::Object_var objVar;
  CosEventComm::PushSupplier_var supplierVar;
  CosEventChannelAdmin::SupplierAdmin_var adminVar;

  ACE_TRY_NEW_ENV
    {
      // Create the supplier reference.
      objVar = pAdapter_->getPOA ()->servant_to_reference (this);
      supplierVar = CosEventComm::PushSupplier::_narrow (objVar.in ());

      // Obtain a proxy.
      adminVar = channel->for_suppliers (ACE_ENV_SINGLE_ARG_PARAMETER);
      ACE_TRY_CHECK;
      proxyVar_ = adminVar->obtain_push_consumer (ACE_ENV_SINGLE_ARG_PARAMETER);
      ACE_TRY_CHECK;

      // Perform the connection.
      proxyVar_->connect_push_supplier (supplierVar.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCH (CosEventChannelAdmin::AlreadyConnected, ac)
    {
      ACE_PRINT_EXCEPTION (ac, ACE_TEXT ("ERROR: Failed to connect -- proxy is already connected\n"));
      exit (1);
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to connect\n"));
      exit (1);
    }
  ACE_ENDTRY;

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Connected supplier to log channel\n")));
}

void
EventLogExample::generate ()
{
  CORBA::Any any;
  char message[30];

  for (int i = 0; i < LOG_EVENTS; i++)
    {
      ACE_OS::sprintf (message, "Event Log message # %d", i);

      any <<= ACE_OS::strdup (message);

      ACE_TRY_NEW_ENV
        {
          proxyVar_->push (any ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;
        }
      ACE_CATCHANY
        {
          ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to push message\n"));
          exit (1);
        }
      ACE_CATCHALL
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to push message\n")));
          exit (1);
        }
      ACE_ENDTRY;
    }

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Generated some log messages\n")));
}

void
EventLogExample::disconnect ()
{
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Disconnecting supplier from log channel\n")));

  ACE_TRY_NEW_ENV
    {
      proxyVar_->disconnect_push_consumer (ACE_ENV_SINGLE_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCH (CORBA::SystemException, se)
    {
      ACE_PRINT_EXCEPTION (se, ACE_TEXT ("ERROR: Failure while disconnecting\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failure while disconnecting\n")));
      exit (1);
    }
  ACE_ENDTRY;

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Disconnected supplier from log channel\n")));
}

void
EventLogExample::disconnect_push_supplier ()
  ACE_THROW_SPEC ((CORBA::SystemException))
{
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Disconnect callback invoked\n")));
}

int main (int argc, char **argv)
{
  ObjectAdapter *pAdapter;
  ACE_Time_Value *pTV;
  CORBA::ORB_var orbVar;
  EventLogExample *pExample;
  DsEventLogAdmin::EventLog_var logVar;

  // Get the orb reference and start the thread (non-blocking).
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Initialising ORB\n")));
  orbVar = CORBA::ORB_init (argc, argv);
  pTV = new ACE_Time_Value (ACE_Time_Value::zero);
  orbVar->run (pTV);

  // Create the example.
  pAdapter = new ObjectAdapter (orbVar);
  pExample = new EventLogExample (orbVar, pAdapter);
  pAdapter->createTransient (pExample);

  // Run the example.
  logVar = pExample->create ();
  pExample->connect (logVar.in ());
  pExample->generate ();
  pExample->disconnect ();

  delete pExample;
  delete pAdapter;

  return 0;
}
