/*
 * Utility class for the naming service bank example.
 */

#include "Util.h"


Util::Util (CORBA::ORB_var orbVar)
  : orbVar_ (orbVar), rootPoaVar_ (0), rootCtxVar_ (0), accMgrCtxVar_ (0), accountsCtxVar_ (0)
{
  // Get a reference to the root POA.
  CORBA::Object_var objVar;
  ACE_TRY_NEW_ENV
    {
      objVar = orbVar_->resolve_initial_references (ACE_TEXT ("RootPOA") ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      rootPoaVar_ = PortableServer::POA::_narrow (objVar.in ());
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to obtain reference to root POA\n"));
      exit (-1);
    }
  ACE_ENDTRY;
  
  // Get a reference to the root context.
  ACE_TRY_NEW_ENV
    {
      objVar = orbVar_->resolve_initial_references (ACE_TEXT ("NameService") ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      rootCtxVar_ = CosNaming::NamingContextExt::_narrow (objVar.in ());
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to obtain a reference to the root context\n"));
      exit (-1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to obtain a reference to the root context\n")));
      exit (-1);
    }
  ACE_ENDTRY;
}

PortableServer::POA_var
Util::getRootPOA ()
{
  return rootPoaVar_;
}

void
Util::buildNameGraph ()
{
  CosNaming::NamingContext_var plainRootVar;
  CosNaming::NamingContext_var acmeCtxVar;

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Building name service graph\n")));

  plainRootVar = CosNaming::NamingContext::_narrow (getRootNamingCtx ().in ());
  acmeCtxVar = myBindContext (plainRootVar.in (), ACE_TEXT ("ACME_Bank"));
  accMgrCtxVar_ = myBindContext (acmeCtxVar.in (), ACE_TEXT ("AccountManager"));
  accountsCtxVar_ = myBindContext (acmeCtxVar.in (), ACE_TEXT ("Accounts"));
}

Bank::AccountManager_ptr
Util::getAccManagerObject ()
{
  CORBA::Object_var objVar;
  Bank::AccountManager_var accManagerVar;

  ACE_TRY_NEW_ENV
    {
      objVar = getRootNamingCtx ()->resolve_str (ACE_TEXT ("ACME_Bank/AccountManager/AccountManagerServer"));

      if (CORBA::is_nil (objVar.in ()))
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: AccountManager is null\n")));
          exit (1);
        }

      accManagerVar = Bank::AccountManager::_narrow (objVar.in ());
     }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to obtain AccountManager\n"));
      exit (1);
    }
  ACE_ENDTRY;

  return accManagerVar._retn ();
}

void
Util::bindAccountManager (CORBA::Object_ptr obj)
{
  CosNaming::Name_var nameVar;
  CORBA::Boolean rebind = 0;

  ACE_TRY_NEW_ENV
    {
      nameVar = getRootNamingCtx ()->to_name (ACE_OS::strdup (ACE_TEXT ("AccountManagerServer")) ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      getAccManagerCtx ()->bind (nameVar.in (), obj ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCH (CosNaming::NamingContext::AlreadyBound, ab)
    {
      rebind = 1;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to bind account manager\n"));
      exit (-1);
    }
  ACE_ENDTRY;
  
  // Handle a rebind needed on a server re-start.
  if (rebind)
    {
      ACE_TRY_NEW_ENV
        {
          getAccManagerCtx ()->rebind (nameVar.in (), obj ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;
        }
      ACE_CATCHANY
        {
          ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to rebind account manager\n"));
          exit (-1);
        }
      ACE_ENDTRY;
    }
    
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Successfully bound account manager to name service\n")));
}

void
Util::addAccount (const char *name, CORBA::Object_ptr obj)
{
  CosNaming::Name_var accNameVar;
  Bank::Account_ptr account = getAccount (name);

  if (CORBA::is_nil (account))
    {
      ACE_TRY_NEW_ENV
        {
          accNameVar = getRootNamingCtx ()->to_name (ACE_OS::strdup (name) ACE_ENV_ARG_PARAMETER);
          getAccountsCtx ()->bind (accNameVar.in (), obj ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;
        }
      ACE_CATCH (CosNaming::NamingContext::AlreadyBound, ab)
        {
          // This is impossible but just in case we'll rebind.
          ACE_DEBUG ((LM_INFO, ACE_TEXT ("WARNING: unexpected rebind when adding account %s\n"), name));
          getAccountsCtx ()->rebind (accNameVar.in (), obj);
        }
      ACE_ENDTRY;
    }
}

Bank::Account_ptr
Util::getAccount (const char *name)
{
  Bank::Account_ptr account;
  CosNaming::Name accName;
  CORBA::Object_var objVar;

  ACE_TRY_NEW_ENV
    {
      accName.length (1);
      accName[0].id = ACE_OS::strdup (name);

      objVar = getAccountsCtx ()->resolve (accName ACE_ENV_ARG_PARAMETER);
      account = Bank::Account::_narrow (objVar.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      account = 0;
    }
  ACE_ENDTRY;

  return account;
}

CosNaming::NamingContext_var Util::getAccManagerCtx ()
{
  CORBA::Object_var objVar;

  if (accMgrCtxVar_ == 0)
    {
      ACE_TRY_NEW_ENV
        {
          objVar = getRootNamingCtx ()->resolve_str (ACE_TEXT ("ACME_Bank/AccountManager") ACE_ENV_ARG_PARAMETER);
          accMgrCtxVar_ = CosNaming::NamingContext::_narrow (objVar.in () ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;
        }
      ACE_CATCHANY
        {
          ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to obtain account manager context\n"));
          exit (-1);
        }
      ACE_ENDTRY;
    }
        
  return accMgrCtxVar_;
}

CosNaming::NamingContext_var Util::getAccountsCtx ()
{
  CORBA::Object_var objVar;

  if (accountsCtxVar_ == 0)
    {
      ACE_TRY_NEW_ENV
        {
          objVar = getRootNamingCtx ()->resolve_str (ACE_TEXT ("ACME_Bank/Accounts") ACE_ENV_ARG_PARAMETER);
          accountsCtxVar_ = CosNaming::NamingContext::_narrow (objVar.in () ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;
        }
      ACE_CATCHANY
        {
          ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to obtain account context\n"));
          exit (-1);
        }
      ACE_ENDTRY;
    }

  return accountsCtxVar_;
}

CosNaming::NamingContextExt_var Util::getRootNamingCtx ()
{
  return rootCtxVar_;
}

CosNaming::NamingContext_ptr
Util::myBindContext (CosNaming::NamingContext_ptr parentCtx, const char *ctxName)
{
  CosNaming::Name_var nameVar;
  CosNaming::NamingContext_var ctxVar;

  ACE_TRY_NEW_ENV
    {
      nameVar = getRootNamingCtx ()->to_name (ACE_OS::strdup (ctxName) ACE_ENV_ARG_PARAMETER);
      ctxVar = parentCtx->bind_new_context (nameVar.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCH (CosNaming::NamingContext::AlreadyBound, ab)
    {
      ctxVar = parentCtx->new_context ();
      parentCtx->rebind_context (nameVar.in (), ctxVar.in ());
    }
  ACE_CATCHANY
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Util::myBindContext -- exception while binding s\n"), ctxName));
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT (""));
      exit (-1);
    }
  ACE_ENDTRY;

  return ctxVar._retn ();
}
