#include "CosNamingC.h"

#include "Naming/video/Naming.h"

#include <tao/corba.h>
#include <ace/OS.h>

#include "Common/ServerC.h"


/*
 * @func This program resolves a reference to the Naming Service
 * and finds a video supplier object in a lookup for a given video
 * title.
 */
int
main (int argc, char **argv)
{
  CORBA::ORB_var orbVar;
  CORBA::Object_var objVar;
  CosNaming::NamingContext_var rootContextVar;
  CosNaming::Name rootName;
  CORBA::Object_ptr server;
  Server::ServiceProvider_var providerVar;
  
  // Get a reference to required utilities.
  Naming names;

  // Initialise the ORB.
  ACE_TRY_NEW_ENV
    {
      orbVar = CORBA::ORB_init (argc, argv ACE_ENV_ARG_PARAMETER);
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("INFO: Initialised ORB\n")));
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Cannot initialise ORB\n"));
      exit (-1);
    }
  ACE_ENDTRY;
    
  // Get a reference to the naming service.
  ACE_TRY_NEW_ENV
    {
      objVar = orbVar->resolve_initial_references (ACE_TEXT ("NameService") ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      rootContextVar = CosNaming::NamingContext::_narrow (objVar.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("Failed to resolve/narrow to the NameService\n"));
      exit (-1);
    }
  ACE_ENDTRY;

  // Prepare name context to service provider binding.
  rootName.length (1);
  rootName[0].id = ACE_OS::strdup (ACE_TEXT ("ServiceProvider"));
  rootName[0].kind = ACE_OS::strdup (ACE_TEXT ("server"));

  // Resolve service provider (server) from naming service.
  names.resolveToObject (&rootName, rootContextVar.in (), &server);

  // Narrow server to ServiceProvider implementation.
  ACE_TRY_NEW_ENV
    {
      providerVar = Server::ServiceProvider::_narrow (server ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Cannot narrow to VideoServiceProvider\n"));
      exit (-1);
    }
  ACE_ENDTRY;
  if (CORBA::is_nil (providerVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Resolved to a null VideoServiceProvider object\n")));
      exit (-1);
    }

  // Retrieve the list of companies from the service provider.
  Server::CompanyNameList *list = providerVar->getCompanies ();
  if (list)
    {
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Available companies:-\n")));
      for (int count = 0; count < (int) list->length (); count++)
        {
          ACE_DEBUG ((LM_INFO, ACE_TEXT ("%s\n"), (const char *) ((*list)[count])));
        }

      // Resolve an arbitary company from the naming service.
      CosNaming::Name companyName;
      companyName.length (3);
      companyName[0].id = ACE_OS::strdup (ACE_TEXT ("Video Stream Supplier"));
      companyName[1].id = ACE_OS::strdup (ACE_TEXT ("Films"));
      companyName[2].id = ACE_OS::strdup (ACE_TEXT ("Videos for Sale"));

      CORBA::Object_ptr myObject = 0;
      names.resolveToObject (&companyName, rootContextVar.in (), &myObject);
      if (CORBA::is_nil (myObject))
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Could not resolve to the chosen company\n")));
          exit (-1);
        }

      Server::Company_var companyVar;
      ACE_TRY_NEW_ENV
        {
          companyVar = Server::Company::_narrow (myObject ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;
        }
      ACE_CATCHANY
        {
          ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Cannot narrow to company\n"));
          exit (-1);
        }
      ACE_ENDTRY;

      if (CORBA::is_nil (companyVar.in ()))
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Narrowed to a null company\n")));
          exit (-1);
        }

      // Retrieve the list of offered films from the company.
      CORBA::String_var catVar = ACE_OS::strdup ("Films");
      Server::OfferList *offers;
      ACE_TRY_NEW_ENV
        {
          offers = companyVar->getOffers (catVar.in () ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;
          if (offers)
            {
              ACE_DEBUG ((LM_INFO, ACE_TEXT ("Available 'Films' feeds:\n")));
              for (int count2 = 0; count2 < (int) offers->length (); count2++)
                {
                  ACE_DEBUG ((LM_INFO, ACE_TEXT ("%s\n"), (const char *) ((*offers)[count2])));
                }
            }
          else
            {
              ACE_DEBUG ((LM_INFO, ACE_TEXT ("Null list of offers returned\n")));
            }
        }
      ACE_CATCHANY
        {
          ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to get offers from company\n"));
          exit (-1);
        }
      ACE_ENDTRY;
    }
  else
    {
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Null list of companies returned\n")));
    }

  // Complete.
  return 0;
}
