/*
 * Utility class for the mail notification service example.  This class
 * provides access to a single event channel created with persistent
 * ConnectionReliability and EventReliability.
 */

#include "ChannelUtil.h"


/*
 * @func Constructs a ChannelUtil object
 * @parm | orbVar | a reference to the ORB
 * @rdesc LM_ERROR message and exit on error
 */
ChannelUtil::ChannelUtil (CORBA::ORB_var orbVar)
  : orbVar_ (orbVar), rootPoaVar_ (0), transientPoaVar_ (0), persistentPoaVar_ (0), channelFactoryVar_ (0), channelVar_ (0)
{
  if (CORBA::is_nil (orbVar_.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Passed a null ORB reference\n")));
      exit (1);
    }
}

/*
 * @func Destroy this object
 */
ChannelUtil::~ChannelUtil ()
{
}

/*
 * @func Returns a reference to the event channel
 * @rdesc Reference to the event channel, or LM_ERROR message and exit on error
 */
CosNotifyChannelAdmin::EventChannel_var
ChannelUtil::getChannel ()
{
  CORBA::Object_var objVar;

  if (channelFactoryVar_ == 0)
    {
      // Obtain a reference to the event channel factory.
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Resolving to notification service\n")));
      objVar = orbVar_->resolve_initial_references (ACE_TEXT ("NotificationService"));
      if (CORBA::is_nil (objVar.in ()))
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Could not resolve to the notification service\n")));
          exit (1);
        }
      channelFactoryVar_ = CosNotifyChannelAdmin::EventChannelFactory::_narrow (objVar.in ());
    }

  // Try to obtain the channel with ID 0.
  if (channelVar_ == 0)
    {
      ACE_TRY_NEW_ENV
        {
          channelVar_ = channelFactoryVar_->get_event_channel (0 ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;

          ACE_DEBUG ((LM_INFO, ACE_TEXT ("INFO: Found channel with id 0\n")));
        }
      ACE_CATCH (CosNotifyChannelAdmin::ChannelNotFound, cnf)
        {
          // No action needed at this point.
        }
      ACE_ENDTRY;
    }

    // If it could not be found, create a new channel.
  if (channelVar_ == 0)
    {
      CosNotifyChannelAdmin::ChannelID id;
      CosNotification::QoSProperties qos = CosNotification::QoSProperties (2);
      CosNotification::AdminProperties adm = CosNotification::AdminProperties (0);

      qos.length (2);
      qos[0].name= CORBA::string_dup (CosNotification::ConnectionReliability);
      qos[0].value <<= CosNotification::Persistent;
      qos[1].name= CORBA::string_dup (CosNotification::EventReliability);
      qos[1].value <<= CosNotification::Persistent;

      ACE_TRY_NEW_ENV
        {
          channelVar_ = channelFactoryVar_->create_channel (qos, adm, id ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;

          ACE_DEBUG ((LM_INFO, ACE_TEXT ("INFO: Created channel with id %i\n"), id));
        }
      ACE_CATCHANY
        {
          ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to create the event channel\n"));
          exit (1);
        }
      ACE_CATCHALL
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to create the event channel\n")));
        }
      ACE_ENDTRY;
    }

  return channelVar_;
}

/*
 * @func Returns a reference to the root POA.
 * @rdesc Reference to the root POA, or LM_ERROR message and exit if an error occurs.
 */
PortableServer::POA_var
ChannelUtil::getRootPoa ()
{
  CORBA::Object_var objVar;

  if (rootPoaVar_ == 0)
    {
      // Obtain a reference to the root POA.
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Resolving to root POA\n")));
      objVar = orbVar_->resolve_initial_references (ACE_TEXT ("RootPOA"));
      if (CORBA::is_nil (objVar.in ()))
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Could not resolve to the root POA\n")));
          exit (1);
        }
      rootPoaVar_ = PortableServer::POA::_narrow (objVar.in ());
    }

  return rootPoaVar_;
}

/*
 * @func Returns a reference to the default transient POA.
 * @rdesc Reference to the default transient POA
 */
PortableServer::POA_var
ChannelUtil::getDefaultTransientPoa ()
{
  if (transientPoaVar_ == 0)
    {
      transientPoaVar_ = createTransientPoa (ACE_TEXT ("ChannelUtilTransientPOA"));
    }

  return transientPoaVar_;
}

/*
 * @func Returns a reference to the default persistent POA.
 * @rdesc Reference to the default persistent POA
 */
PortableServer::POA_var
ChannelUtil::getDefaultPersistentPoa ()
{
  if (persistentPoaVar_ == 0)
    {
      persistentPoaVar_ = createPersistentPoa (ACE_TEXT ("ChannelUtilPersistentPOA"));
    }

  return persistentPoaVar_;
}

/*
 * @func Creates a transient POA (with the root POA as parent).
 * @parm | name | the adapter name for the transient POA
 * @rdesc Reference to the transient POA
 */
PortableServer::POA_var
ChannelUtil::createTransientPoa (const char *name)
{
  return createPoa (name, 0);
}

/*
 * @func Creates a persistent POA (with the root POA as parent).
 * @parm | name | the adapter name for the persistent POA
 * @rdesc Reference to the persistent POA
 */
PortableServer::POA_var
ChannelUtil::createPersistentPoa (const char *name)
{
  return createPoa (name, 1);
}

/*
 * @func Creates a child POA from the root POA, which will use the root POA's manager.
 * @parm | name | the adapter name for the POA
 * @parm | persistent | indicates if the POA is to be persistent (true) or transient (false)
 * @rdesc Reference to the child POA
 */
PortableServer::POA_var
ChannelUtil::createPoa (const char *name, CORBA::Boolean persistent)
{
  PortableServer::POA_var poaVar;

  // Ensure the root POA reference is valid.
  if (rootPoaVar_ == 0)
    {
      getRootPoa ();
    }

  // Declare the required policies.
  CORBA::PolicyList policies;
  policies.length (2);
  PortableServer::LifespanPolicyValue lifespan;
  PortableServer::IdAssignmentPolicyValue idAssignment;

  // Set the policies depending on the type of POA required.
  lifespan =     (persistent ? PortableServer::PERSISTENT : PortableServer::TRANSIENT);
  idAssignment = (persistent ? PortableServer::USER_ID    : PortableServer::SYSTEM_ID);
  policies[0] = rootPoaVar_->create_lifespan_policy (lifespan);
  policies[1] = rootPoaVar_->create_id_assignment_policy (idAssignment);

  // Create the POA and activate the manager.
  poaVar = rootPoaVar_->create_POA (ACE_OS::strdup (name), rootPoaVar_->the_POAManager (), policies);
  poaVar->the_POAManager ()->activate ();

  return poaVar;
}

/*
 * @func Checks to see if a file exists
 * @parm | filename | the name of the file
 * @rdesc Boolean indicating if the file exists or not
 */
CORBA::Boolean
ChannelUtil::fileExists (const char *filename)
{
  FILE *f = ACE_OS::fopen (filename, ACE_TEXT ("r"));
  if (f == 0)
    {
      // Failed to open file.
      return 0;
    }

  // File exists and was opened, so close it.
  ACE_OS::fclose (f);
  return 1;
}

/*
 * @func Deletes a file
 * @parm | filename | the name of the file
 */
void
ChannelUtil::deleteFile (const char *filename)
{
  ACE_OS::unlink (filename);
}

/*
 * @func Writes a CORBA object's IOR out to a file
 * @parm | filename | the name of the file for the IOR
 * @parm | obj | the CORBA object
 * @rdesc LM_ERROR message if  the file could not be written
 */
void
ChannelUtil::writeIOR (const char *filename, CORBA::Object_ptr obj)
{
  char *ior = orbVar_->object_to_string (obj);
  FILE *output_file = ACE_OS::fopen (filename, ACE_TEXT ("w"));
  if (output_file == 0)
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Could not open output file for writing IOR: %s\n"), filename));
      return;
    }

  ACE_OS::fprintf (output_file, ACE_TEXT ("%s"), ior);
  ACE_OS::fclose (output_file);
}

/*
 * @func Creates a CORBA object reference from an IOR in a file
 * @parm | filename | the name of the file with the IOR
 * @rdesc The CORBA object reference, or LM_ERROR message if object could not be created
 */
CORBA::Object_ptr
ChannelUtil::readIOR (const char *filename)
{
  CORBA::Object_var objVar;

  FILE *input_file = ACE_OS::fopen (filename, ACE_TEXT ("r"));
  if (input_file == 0)
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Could not open input file for reading IOR: %s\n"), filename));
      return 0;
    }

  char ior[1024];
  ACE_OS::fgets (ior, 1024, input_file);
  ACE_OS::fclose (input_file);

  ACE_TRY_NEW_ENV
    {
      objVar = orbVar_->string_to_object (ior ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("Failed to convert object reference from the IOR\n"));
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("File: %s, IOR: %s\n"), filename, ior));
      return 0;
    }
  ACE_ENDTRY;

  if (CORBA::is_nil (objVar.in ()))
    {
      ACE_DEBUG ((LM_DEBUG, ACE_TEXT ("Could not get valid object reference from IOR in file: %s\n"), filename));
      return 0;
    }

  return objVar._retn ();
}
