/*
 * The MailBoxImpl class implements the MailBox interface defined in Mail.idl.  This
 * class acts as a user's inbox and outbox.  The user can use the MailTool utility to
 * access their instance of MailBoxImpl, and use this instance to send "mail" to and
 * receive "mail" from users over the OpenFusion Notification Service.
 */

#include "MailBoxImpl.h"


/*
 * @func Constructs a MailBoxImpl object
 * @parm | orbVar | a reference to the ORB
 * @parm | pUtil | a reference to the channel utility
 * @parm | name | the name of the mail box
 * @rdesc LM_ERROR message and exit on error
 */
MailBoxImpl::MailBoxImpl (CORBA::ORB_var orbVar, ChannelUtil *pUtil, const char *name)
: orbVar_ (orbVar), pUtil_ (pUtil), username_ (0), poaVar_ (0), oidVar_ (0), inboxProxyVar_ (0), outboxProxyVar_ (0)
{
  PortableServer::ServantBase_var servantVar;
  char iorFileName[64];

  username_ = ACE_OS::strdup (name);

  // If the IOR file exists, then the mailbox already exists.
  ACE_OS::strcpy (iorFileName, username_);
  ACE_OS::strcat (iorFileName, ACE_TEXT (".ior"));
  if (pUtil_->fileExists (iorFileName))
    {
      throw Mail::AccountExists ();
    }

  // Obtain a reference to a transient POA.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Obtaining default transient POA\n"))); 
  poaVar_ = pUtil_->getDefaultTransientPoa ();

  // Create the mail box CORBA object.
  servantVar = (PortableServer::ServantBase_var) this;
  oidVar_ = poaVar_->activate_object (servantVar.in ());
  CORBA::Object_var thisRefVar = poaVar_->id_to_reference (oidVar_.in ());

  // Write out the mail box's IOR.
  pUtil_->writeIOR (iorFileName, thisRefVar.in ());

  // Obtain a reference to the channel.
  CosNotifyChannelAdmin::EventChannel_var channelVar = pUtil_->getChannel ();

  // Structured pull consumer: obtain and connect to a structured proxy pull supplier.
  CosNotifyComm::StructuredPullConsumer_var consumerVar;
  CosNotifyChannelAdmin::ProxySupplier_var psVar;
  CosNotifyChannelAdmin::ClientType type = CosNotifyChannelAdmin::STRUCTURED_EVENT;
  CosNotifyChannelAdmin::ProxyID pid;
  CosNotifyChannelAdmin::ConsumerAdmin_var consAdminVar;
  ACE_TRY_NEW_ENV
    {
      consAdminVar = channelVar->default_consumer_admin (ACE_ENV_SINGLE_ARG_PARAMETER);
      psVar = consAdminVar->obtain_notification_pull_supplier (type, pid ACE_ENV_ARG_PARAMETER);
      inboxProxyVar_ = CosNotifyChannelAdmin::StructuredProxyPullSupplier::_narrow (psVar.in () ACE_ENV_ARG_PARAMETER);

      consumerVar = CosNotifyComm::StructuredPullConsumer::_narrow (thisRefVar.in () ACE_ENV_ARG_PARAMETER);
      inboxProxyVar_->connect_structured_pull_consumer (consumerVar.in () ACE_ENV_ARG_PARAMETER);

      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("Failed to connect as a structured pull consumer\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to connect as a structured pull consumer\n")));
      exit (1);
    }
  ACE_ENDTRY;

  // Structured push supplier: obtain and connect to a structured proxy push consumer.
  CosNotifyComm::StructuredPushSupplier_var supplierVar;
  CosNotifyChannelAdmin::ProxyConsumer_var pcVar;
  CosNotifyChannelAdmin::SupplierAdmin_var suppAdminVar;
  ACE_TRY_NEW_ENV
    {
      suppAdminVar = channelVar->default_supplier_admin (ACE_ENV_SINGLE_ARG_PARAMETER);
      pcVar = suppAdminVar->obtain_notification_push_consumer (type, pid ACE_ENV_ARG_PARAMETER);
      outboxProxyVar_ = CosNotifyChannelAdmin::StructuredProxyPushConsumer::_narrow (pcVar.in () ACE_ENV_ARG_PARAMETER);

      supplierVar = CosNotifyComm::StructuredPushSupplier::_narrow (thisRefVar.in () ACE_ENV_ARG_PARAMETER);
      outboxProxyVar_->connect_structured_push_supplier (supplierVar.in () ACE_ENV_ARG_PARAMETER);

      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("Failed to connect as a structured push consumer\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to connect as a structured push consumer\n")));
      exit (1);
    }
  ACE_ENDTRY;

  // Filter the "inbox" so that only events with the username in the recipients are received.
  CosNotifyFilter::FilterFactory_var factoryVar = channelVar->default_filter_factory ();
  CosNotifyFilter::Filter_var filterVar;
  ACE_TRY_NEW_ENV
    {
      // Create the filter.
      filterVar = factoryVar->create_filter (ACE_TEXT ("EXTENDED_TCL") ACE_ENV_ARG_PARAMETER);

      // Create the constraint expression.
      char constraint[64];
      ACE_OS::strcpy (constraint, ACE_TEXT ("\'"));
      ACE_OS::strcat (constraint, name);
      ACE_OS::strcat (constraint, ACE_TEXT ("\' in $recipients"));

      // Create the constraint and add it to the filter.
      CosNotification::EventType type;
      type.domain_name = ACE_OS::strdup (ACE_TEXT ("Mail"));
      type.type_name = ACE_OS::strdup (ACE_TEXT ("Message"));
      CosNotification::EventTypeSeq evts;
      evts.length (1);
      evts[0] = type;
      CosNotifyFilter::ConstraintExp exp;
      exp.event_types = evts;
      exp.constraint_expr = ACE_OS::strdup (constraint);
      CosNotifyFilter::ConstraintExpSeq exps;
      exps.length (1);
      exps[0] = exp;
      filterVar->add_constraints (exps ACE_ENV_ARG_PARAMETER);

      // Add the filter to the proxy.
      inboxProxyVar_->add_filter (filterVar.in () ACE_ENV_ARG_PARAMETER);

      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("Failed to set a filter on the inbox proxy\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, "Failed to set a filter on the inbox proxy\n"));
      exit (1);
    }
  ACE_ENDTRY;
}

/*
 * @func Destroy this object
 */
MailBoxImpl::~MailBoxImpl ()
{
}

/*
 * @func Sends a message
 * @parm message | the message to send
 * @parm to | string sequence of recipients
 * @rdesc LM_ERROR message on error
 */
void
MailBoxImpl::sendMessage (const char *message, const Mail::StringSeq &to)
  ACE_THROW_SPEC ((CORBA::SystemException))
{
  CosNotification::StructuredEvent event;

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Send called\n")));

  // Header.
  CosNotification::EventType type;
  type.domain_name = ACE_OS::strdup (ACE_TEXT ("Mail"));
  type.type_name = ACE_OS::strdup (ACE_TEXT ("Message"));
  CosNotification::FixedEventHeader fixedHeader;
  fixedHeader.event_type = type;
  fixedHeader.event_name = ACE_OS::strdup (ACE_TEXT (""));
  CosNotification::PropertySeq variableHeader;
  variableHeader.length (0);
  CosNotification::EventHeader header;
  header.fixed_header = fixedHeader;
  header.variable_header = variableHeader;
  event.header = header;

  // Filterable data.
  // 1: list of recipients.  This allows filtering to be performed, so that
  //    only the requested recipients receive a given message.
  // 2: the username of the sending mailbox.
  CosNotification::PropertySeq filterable;
  filterable.length (2);
  filterable[0].name = ACE_OS::strdup (ACE_TEXT ("recipients"));
  filterable[0].value <<= to;
  filterable[1].name = ACE_OS::strdup (ACE_TEXT ("sender"));
  filterable[1].value <<= ACE_OS::strdup (username_);
  event.filterable_data = filterable;

  // Remainder-of-body: the message.
  event.remainder_of_body <<= ACE_OS::strdup (message);

  // Push the event
  ACE_TRY_NEW_ENV
    {
      outboxProxyVar_->push_structured_event (event ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Successfully sent event\n")));
    }
  ACE_CATCH (CosEventComm::Disconnected, d)
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to send event: user is disconnected from proxy\n")));
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("Failed to send event\n"));
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to send event\n")));
    }
  ACE_ENDTRY;
}

/*
 * @func Receives a message, if one is waiting for this user
 * @rdesc true or false indicating if a message was received: if true, sender and message
 *        are populated with the appropriate information
 */
CORBA::Boolean
MailBoxImpl::receiveMessage (CORBA::String_out sender, CORBA::String_out message)
  ACE_THROW_SPEC ((CORBA::SystemException))
{
  CosNotification::StructuredEvent *event;
  CORBA::Boolean hasEvent = 0;
  const char *val;

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Receive called\n")));

  // Try to pull an event from the inbox proxy.
  ACE_TRY_NEW_ENV
    {
      event = inboxProxyVar_->try_pull_structured_event (hasEvent ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCH (CosEventComm::Disconnected, dis)
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Could not receive events: mailbox is not connected to the proxy\n")));
      return 0;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("Failed to receive event from proxy pull supplier\n"));
      return 0;
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to receive event from proxy pull supplier\n")));
      return 0;
    }
  ACE_ENDTRY;

  // Return if there was no event to be pulled.
  if (! hasEvent)
    {
      return hasEvent;
    }

  // Extract the information from the event.
  ACE_TRY_NEW_ENV
    {
      // Extract the sender information.
      CosNotification::PropertySeq filt = event->filterable_data;
      for (unsigned int i = 0; i < filt.length () ;i++)
        {
          if (ACE_OS::strcmp (filt[i].name, ACE_TEXT ("sender")) == 0)
            {
              filt[i].value >>= val;
              sender = CORBA::string_dup (val);
              break;
            }
        }

      // Extract the message information.
      event->remainder_of_body >>= val;
      message = CORBA::string_dup (val);

      ACE_TRY_CHECK;
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Received event, but could not access sender/message information\n")));
      return 0;
    }
  ACE_ENDTRY;

  return hasEvent;
}

/*
 * @func Destroys this mailbox
 * @rdesc LM_ERROR message if an exception occurs
 */
void
MailBoxImpl::destroy ()
  ACE_THROW_SPEC ((CORBA::SystemException))
{
  ACE_TRY_NEW_ENV
    {
      // Disconnect from the proxies.
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Disconnecting the mailbox from the proxies\n")));
      inboxProxyVar_->disconnect_structured_pull_supplier (ACE_ENV_SINGLE_ARG_PARAMETER);
      outboxProxyVar_->disconnect_structured_push_consumer (ACE_ENV_SINGLE_ARG_PARAMETER);

      // Delete the IOR file for this mailbox.
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Deleting the mailbox IOR file\n")));
      char filename[64];
      ACE_OS::strcpy (filename, username_);
      ACE_OS::strcat (filename, ACE_TEXT (".ior"));
      pUtil_->deleteFile (filename);

      // Destroy the CORBA object for this mailbox.
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Deactivating the mailbox's CORBA object\n")));
      poaVar_->deactivate_object (oidVar_.in ());

      ACE_TRY_CHECK;

      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Successfully deleted account for %s\n"), username_));
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("Failed to delete account\n"));
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to delete account for %s\n"), username_));
    }
  ACE_ENDTRY;
}

void
MailBoxImpl::offer_change (const CosNotification::EventTypeSeq & /*added*/, const CosNotification::EventTypeSeq & /*removed*/)
  ACE_THROW_SPEC ((CORBA::SystemException,CosNotifyComm::InvalidEventType))
{
}

void
MailBoxImpl::subscription_change (const CosNotification::EventTypeSeq & /*added*/, const CosNotification::EventTypeSeq & /*removed*/)
  ACE_THROW_SPEC ((CORBA::SystemException,CosNotifyComm::InvalidEventType))
{
}

void
MailBoxImpl::disconnect_structured_pull_consumer ()
  ACE_THROW_SPEC ((CORBA::SystemException))
{
}

void
MailBoxImpl::disconnect_structured_push_supplier ()
  ACE_THROW_SPEC ((CORBA::SystemException))
{
}
