/*
 * A simple mail server.
 */

#include "MailServerImpl.h"


/*
 * @func Constructs a MailServerImpl object
 * @parm | orbVar | a reference to the ORB
 * @parm | pUtil | a reference to the channel utility
 * @rdesc
 */
MailServerImpl::MailServerImpl (CORBA::ORB_var orbVar, ChannelUtil *pUtil)
  : orbVar_ (orbVar), pUtil_ (pUtil)
{
  PortableServer::ServantBase_var servantVar;
  PortableServer::ObjectId_var oidVar;
  CORBA::Object_var objVar;

  // Obtain a reference to a transient POA.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Obtaining default transient POA\n"))); 
  PortableServer::POA_var poaVar = pUtil_->getDefaultTransientPoa ();

  // Create the mail server CORBA object.
  servantVar = (PortableServer::ServantBase_var) this;
  oidVar = poaVar->activate_object (servantVar.in ());
  objVar = poaVar->id_to_reference (oidVar.in ());

  // Write out the mail server's IOR.
  pUtil_->writeIOR (ACE_TEXT ("MailServer.ior"), objVar.in ());
}

/*
 * @func Destroys this object
 */
MailServerImpl::~MailServerImpl ()
{
}

/*
 * @func Creates an account (mailbox) for a user
 * @parm | name | the user name for the account
 * @rdesc AccountExists is thrown if an account with the name already exists
 */
void
MailServerImpl::createAccount (const char *name)
  ACE_THROW_SPEC ((Mail::AccountExists))
{
  MailBoxImpl *mailBox;

  // Create the new mailbox.
  mailBox = new MailBoxImpl (orbVar_, pUtil_, name);
}

/*
 * @func Deletes an account (mailbox) for a user
 * @parm | name | the user name for the account
 * @rdesc NoSuchAccount is thrown if an account with the name does not exist
 */
void
MailServerImpl::deleteAccount (const char *name)
  ACE_THROW_SPEC ((Mail::NoSuchAccount))
{
  CORBA::Object_var objVar;
  Mail::MailBox_var mailBoxVar;
  char filename[64];

  ACE_OS::strcpy (filename, name);
  ACE_OS::strcat (filename, ACE_TEXT (".ior"));
  if (! pUtil_->fileExists (filename))
    {
      ACE_THROW (Mail::NoSuchAccount ());
    }

  objVar = pUtil_->readIOR (filename);
  mailBoxVar = Mail::MailBox::_narrow (objVar.in ());
  mailBoxVar->destroy ();
}

int
main (int argc, char **argv)
{
  CORBA::ORB_var orbVar;
  ChannelUtil *pUtil;
  MailServerImpl *pMailServerImpl;

  // Initialise the ORB reference.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Creating ORB reference\n"))); 
  orbVar = CORBA::ORB_init (argc, argv);
  if (CORBA::is_nil (orbVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Could not create the ORB\n")));
      exit (1);
    }

  // Obtain a reference to the channel utility.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Creating channel utility\n"))); 
  pUtil = new ChannelUtil (orbVar);

  // Create the mail server.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Creating mail server\n")));
  pMailServerImpl = new MailServerImpl (orbVar, pUtil);

  // Handle incoming requests.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Waiting for incoming requests\n"))); 
  orbVar->run ();

  // Clean up and exit.
  delete pMailServerImpl;
  delete pUtil;
  return 0;
}
