/*
 * Base class for all news publishers.
 */

#include "NewsPublisher.h"


NewsPublisher::NewsPublisher (CORBA::ORB_var orbVar)
  : orbVar_ (orbVar), pUtil_ (0), supplierVar_ (0), proxyVar_ (0), eventCount_ (100), maxDelay_ (0L)
{
  if (CORBA::is_nil (orbVar_.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Passed a null ORB reference\n")));
      exit (1);
    }

  // Obtain a reference to the root POA.
  CORBA::Object_var objVar = orbVar_->resolve_initial_references (ACE_TEXT ("RootPOA"));
  PortableServer::POA_var rootPoaVar = PortableServer::POA::_narrow (objVar.in ());

  // Obtain a reference to the POA manager and activate it.
  PortableServer::POAManager_var poaManagerVar = rootPoaVar->the_POAManager ();
  poaManagerVar->activate ();

  // Create the CORBA object for this publisher.
  objVar = _this ();
  supplierVar_ = CosNotifyComm::StructuredPushSupplier::_narrow (objVar.in ());

  // Obtain a reference to the channel utility.
  pUtil_ = new ChannelUtil (orbVar_);

  // Seed the random number generator.
  srand ((unsigned) time (0)); 
}

NewsPublisher::~NewsPublisher ()
{
}

void
NewsPublisher::connectToChannel ()
{
  // Obtain a reference to the channel.
  CosNotifyChannelAdmin::EventChannel_var channelVar = pUtil_->getChannel ();

  // Obtain a reference to the default supplier admin.
  CosNotifyChannelAdmin::SupplierAdmin_var adminVar;
  ACE_TRY_NEW_ENV
    {
      adminVar = channelVar->default_supplier_admin (ACE_ENV_SINGLE_ARG_PARAMETER);
      ACE_TRY_CHECK;
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Obtained default supplier admin\n")));
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to obtain default supplier admin\n")));
      exit (1);
    }
  ACE_ENDTRY;

  CosNotifyChannelAdmin::ProxyID pid;
  CosNotifyChannelAdmin::ClientType type = CosNotifyChannelAdmin::STRUCTURED_EVENT;
  CosNotifyChannelAdmin::ProxyConsumer_var pcVar;

  // Obtain a proxy consumer.
  ACE_TRY_NEW_ENV
    {
      pcVar = adminVar->obtain_notification_push_consumer (type, pid ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      if (CORBA::is_nil (pcVar.in ()))
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Obtained a null proxy consumer\n")));
          exit (1);
       }
     }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to obtain a proxy consumer\n")));
      exit (1);
    }
  ACE_ENDTRY;

  // Narrow to a structured proxy push consumer.
  proxyVar_ = CosNotifyChannelAdmin::StructuredProxyPushConsumer::_narrow (pcVar.in ());
  if (proxyVar_.in () == 0)
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Narrowed to a null structured proxy push consumer\n")));
      exit (1);
    }

  // Connect.
  ACE_TRY_NEW_ENV
    {
      proxyVar_->connect_structured_push_supplier (supplierVar_.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Connected supplier to proxy\n")));
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to connect supplier to proxy\n")));
      exit (1);
    }
  ACE_ENDTRY;
}

void
NewsPublisher::broadcast ()
{
  char message[60];

  for (int i = 0; i < eventCount_; i++)
    {
      CosNotification::StructuredEvent event = createNextEvent ();
      ACE_TRY_NEW_ENV
        {
          proxyVar_->push_structured_event (event ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;

          ACE_OS::memset (message, sizeof (message), sizeof (char));
          pUtil_->printEvent (event, message);
          ACE_DEBUG ((LM_INFO, ACE_TEXT ("Sent message: %s\n"), message));

          delay ();
        }
      ACE_CATCHALL
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to send event\n")));
          break;
        }
      ACE_ENDTRY;
   }
}

void
NewsPublisher::delay ()
{
  long delay = rand () % maxDelay_; 
  ACE_Time_Value atv;
  atv.msec (delay);

  ACE_OS::sleep (atv);
}

void
NewsPublisher::disconnect_structured_push_supplier ()
  ACE_THROW_SPEC ((CORBA::SystemException))
{
}

void
NewsPublisher::subscription_change (const CosNotification::EventTypeSeq & /*added*/, const CosNotification::EventTypeSeq & /*removed*/)
  ACE_THROW_SPEC ((CosNotifyComm::InvalidEventType, CORBA::SystemException))
{
}
