/*
 * Base class for all news subscribers.
 */

#include "NewsSubscriber.h"


NewsSubscriber::NewsSubscriber (CORBA::ORB_var orbVar)
  : orbVar_ (orbVar), channelVar_ (0), consumerVar_ (0), proxyVar_ (0)
{
  if (CORBA::is_nil (orbVar_.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Passed a null ORB reference\n")));
      exit (1);
    }

  // Obtain a reference to the channel utility.
  pUtil_ = new ChannelUtil (orbVar_);

  // Obtain a reference to the root POA.
  CORBA::Object_var objVar = orbVar_->resolve_initial_references (ACE_TEXT ("RootPOA"));
  PortableServer::POA_var poaVar = PortableServer::POA::_narrow (objVar.in ());

  // Obtain a reference to the POA manager and activate it.
  PortableServer::POAManager_var poaManagerVar = poaVar->the_POAManager ();
  poaManagerVar->activate ();

  // Create the CORBA object for this subscriber.
  objVar = _this ();
  consumerVar_ = CosNotifyComm::StructuredPushConsumer::_narrow (objVar.in ());
}

NewsSubscriber::~NewsSubscriber ()
{
  delete pUtil_;
}

void
NewsSubscriber::connectToChannel ()
{
  // Obtain a reference to the channel.
  channelVar_ = pUtil_->getChannel ();

  // Obtain a reference to the default consumer admin.
  CosNotifyChannelAdmin::ConsumerAdmin_var adminVar;
  ACE_TRY_NEW_ENV
    {
      adminVar = channelVar_->default_consumer_admin (ACE_ENV_SINGLE_ARG_PARAMETER);
      ACE_TRY_CHECK;
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Obtained default consumer admin\n")));
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to obtain default consumer admin\n")));
      exit (1);
    }
  ACE_ENDTRY;

  CosNotifyChannelAdmin::ProxyID pid;
  CosNotifyChannelAdmin::ClientType type = CosNotifyChannelAdmin::STRUCTURED_EVENT;
  CosNotifyChannelAdmin::ProxySupplier_var psVar;

  // Obtain a proxy supplier.
  ACE_TRY_NEW_ENV
    {
      psVar = adminVar->obtain_notification_push_supplier (type, pid ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      if (CORBA::is_nil (psVar.in ()))
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Obtained a null proxy supplier\n")));
          exit (1);
       }
     }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to obtain a proxy supplier\n")));
      exit (1);
    }
  ACE_ENDTRY;

  // Narrow to a structured proxy push supplier.
  proxyVar_ = CosNotifyChannelAdmin::StructuredProxyPushSupplier::_narrow (psVar.in ());
  if (proxyVar_.in () == 0)
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Narrowed to a null structured proxy push supplier\n")));
      exit (1);
    }

  // Connect.
  ACE_TRY_NEW_ENV
    {
      proxyVar_->connect_structured_push_consumer (consumerVar_.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Connected consumer to proxy\n")));
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to connect consumer to proxy\n")));
      exit (1);
    }
  ACE_ENDTRY;
}

void
NewsSubscriber::setFilter (const char *domainName, const char *typeName, const char *constraint)
{
  if (CORBA::is_nil (proxyVar_.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("No proxy push supplier\n")));
      exit (1);
    }

  // Obtain a reference to the filter factory.
  CosNotifyFilter::FilterFactory_var factoryVar = channelVar_->default_filter_factory ();
  if (CORBA::is_nil (factoryVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, "Cannot get default filter factory\n"));
      exit (1);
    }

  const char *grammar = ACE_TEXT ("EXTENDED_TCL");
  CosNotifyFilter::Filter_var filterVar;
  ACE_TRY_NEW_ENV
    {
      // Create the filter.
      filterVar = factoryVar->create_filter (grammar ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      // Create the constraint and add it to the filter.
      CosNotification::EventType type;
      type.domain_name = ACE_OS::strdup (domainName);
      type.type_name = ACE_OS::strdup (typeName);
      CosNotification::EventTypeSeq evts;
      evts.length (1);
      evts[0] = type;
      CosNotifyFilter::ConstraintExp exp;
      exp.event_types = evts;
      exp.constraint_expr = ACE_OS::strdup (constraint);
      CosNotifyFilter::ConstraintExpSeq exps;
      exps.length (1);
      exps[0] = exp;
      filterVar->add_constraints (exps ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      // Add the filter to the proxy.
      proxyVar_->add_filter (filterVar.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("Failed to create/add filter\n")));
      exit (1);
    }
  ACE_ENDTRY;
}

void
NewsSubscriber::disconnect_structured_push_consumer ()
  ACE_THROW_SPEC ((CORBA::SystemException))
{
}

void
NewsSubscriber::offer_change (const CosNotification::EventTypeSeq & /*added*/, const CosNotification::EventTypeSeq & /*removed*/)
  ACE_THROW_SPEC ((CosNotifyComm::InvalidEventType, CORBA::SystemException))
{
}

void
NewsSubscriber::push_structured_event (const CosNotification::StructuredEvent& event)
  ACE_THROW_SPEC ((CosEventComm::Disconnected,CORBA::SystemException))
{
  char message[60];
  pUtil_->printEvent (event, message);

  ACE_DEBUG ((LM_INFO, "Received message: %s\n", message));
}
