/*
 * Utility class for the simple notification service example.  This class
 * provides access to a single event channel created with persistent
 * ConnectionReliability and EventReliability.
 */

#include "ChannelUtil.h"


/*
 * @func Constructs a ChannelUtil object
 * @parm | orbVar | a reference to the ORB
 * @rdesc LM_ERROR message and exit on error
 */
ChannelUtil::ChannelUtil (CORBA::ORB_var orbVar)
  : orbVar_ (orbVar)
{
  if (CORBA::is_nil (orbVar_.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Passed a null ORB reference\n")));
      exit (1);
    }
}

/*
 * @func Destroy this object
 */
ChannelUtil::~ChannelUtil ()
{
}

/*
 * @func Returns a reference to the event channel
 * @rdesc Reference to the event channel, or LM_ERROR message and exit on error
 */
CosNotifyChannelAdmin::EventChannel_ptr
ChannelUtil::getChannel ()
{
  CORBA::Object_var objVar;
  CosNotifyChannelAdmin::EventChannelFactory_var channelFactoryVar;
  CosNotifyChannelAdmin::EventChannel_var channelVar;
  
  // Obtain a reference to the event channel factory.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Resolving to notification service\n")));
  objVar = orbVar_->resolve_initial_references (ACE_TEXT ("NotificationService"));
  if (CORBA::is_nil (objVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Could not resolve to the notification service\n")));
      exit (1);
    }
  channelFactoryVar = CosNotifyChannelAdmin::EventChannelFactory::_narrow (objVar.in ());

  // Try to obtain the channel with ID 0.
  ACE_TRY_NEW_ENV
    {
      channelVar = channelFactoryVar->get_event_channel (0 ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;

      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Found channel with id 0\n")));
    }
  ACE_CATCH (CosNotifyChannelAdmin::ChannelNotFound, cnf)
    {
      // No action needed at this point.
    }
  ACE_ENDTRY;

  // If it could not be found, create a new channel.
  if (channelVar == 0)
    {
      CosNotifyChannelAdmin::ChannelID id;
      CosNotification::QoSProperties qos = CosNotification::QoSProperties (2);
      CosNotification::AdminProperties adm = CosNotification::AdminProperties (0);

      qos.length (2);
      qos[0].name= CORBA::string_dup (CosNotification::ConnectionReliability);
      qos[0].value <<= CosNotification::Persistent;
      qos[1].name= CORBA::string_dup (CosNotification::EventReliability);
      qos[1].value <<= CosNotification::Persistent;

      ACE_TRY_NEW_ENV
        {
          channelVar = channelFactoryVar->create_channel (qos, adm, id ACE_ENV_ARG_PARAMETER);
          ACE_TRY_CHECK;

          ACE_DEBUG ((LM_INFO, ACE_TEXT ("Created channel with id %i\n"), id));
        }
      ACE_CATCHANY
        {
          ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to create the event channel\n"));
          exit (1);
        }
      ACE_CATCHALL
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to create the event channel\n")));
        }
      ACE_ENDTRY;
    }

  return channelVar._retn ();
}

/*
 * @func Returns a reference to the root POA.
 * @rdesc Reference to the root POA, or LM_ERROR message and exit if an error occurs.
 */
PortableServer::POA_ptr
ChannelUtil::getRootPoa ()
{
  CORBA::Object_var objVar;
  PortableServer::POA_var rootPoaVar;

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Resolving to root POA\n")));
  objVar = orbVar_->resolve_initial_references (ACE_TEXT ("RootPOA"));
  if (CORBA::is_nil (objVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Could not resolve to the root POA\n")));
      exit (1);
    }

  rootPoaVar = PortableServer::POA::_narrow (objVar.in ());
  rootPoaVar->the_POAManager ()->activate ();

  return rootPoaVar._retn ();
}

/*
 * @func Creates a structured event containing the specified number
 * @parm | count | number to store in the event
 * @rdesc The structured event
 */
CosNotification::StructuredEvent *
ChannelUtil::createStructuredEvent (const int count)
{
  CosNotification::StructuredEvent_var eventVar = new CosNotification::StructuredEvent;

  // Header.
  CosNotification::EventType type;
  type.domain_name = ACE_OS::strdup (ACE_TEXT ("TestDomain"));
  type.type_name = ACE_OS::strdup (ACE_TEXT ("TestType"));
  CosNotification::FixedEventHeader fixedHeader;
  fixedHeader.event_name = ACE_OS::strdup (ACE_TEXT ("TestEvent"));
  fixedHeader.event_type = type;
  CosNotification::PropertySeq variableHeader;
  variableHeader.length (0);
  CosNotification::EventHeader header;
  header.fixed_header = fixedHeader;
  header.variable_header = variableHeader;
  eventVar->header = header;

  // Filterable data.
  CosNotification::PropertySeq filterable;
  filterable.length (0);
  eventVar->filterable_data = filterable;

  // Remainder-of-body.
  eventVar->remainder_of_body <<= count;

  return eventVar._retn ();
}

/*
 * @func Creates an Any event containing the specified number
 * @parm | count | number to store in the event
 * @rdesc The Any event
 */
CORBA::Any_ptr
ChannelUtil::createAnyEvent (const int count)
{
  CORBA::Any_var eventVar = new CORBA::Any;

  eventVar <<= count;
  return eventVar._retn ();
}

/*
 * @func Prints the details of a structured event to a buffer
 * @parm | buffer | a suitably-allocated buffer
 * @parm | event | the structured event to display
 * @parm | full | boolean flag: true to display all event information, false to display only the body
 */
void
ChannelUtil::printEvent (char *buffer, const CosNotification::StructuredEvent &event, CORBA::Boolean full)
{
  // Header information.
  if (full)
    {
      ACE_OS::strcpy (buffer, "\nFIXED HEADER:\n");
      ACE_OS::strcat (buffer, "Domain: <");
      ACE_OS::strcat (buffer, event.header.fixed_header.event_type.domain_name);
      ACE_OS::strcat (buffer, ">, Type: <");
      ACE_OS::strcat (buffer, event.header.fixed_header.event_type.type_name);
      ACE_OS::strcat (buffer, ">, Event: <");
      ACE_OS::strcat (buffer, event.header.fixed_header.event_name);
      ACE_OS::strcat (buffer, ">\n");

      ACE_OS::strcat (buffer, "VARIABLE HEADER:\n");
      printPropertySeq (buffer, event.header.variable_header);
    }

  // Filterable data information.
  if (full)
    {
      ACE_OS::strcat (buffer, "FILTERABLE DATA:\n");
      printPropertySeq (buffer, event.filterable_data);
    }

  // Remainder-of-body information.
  if (full)
    {
      ACE_OS::strcat (buffer, "REMAINDER-OF-BODY:\n");
      printAny (buffer, event.remainder_of_body);
    }
  else
    {
      ACE_OS::strcpy (buffer, "Body: ");
      printAny (buffer, event.remainder_of_body);
    }
}

/*
 * @func Prints the details of an Any event to a buffer
 * @parm | buffer | a suitably-allocated buffer
 * @parm | event | the Any event to display
 * @parm | full | boolean flag: true to display all event information, false to display only the body
 */
void
ChannelUtil::printEvent (char *buffer, const CORBA::Any &event, CORBA::Boolean full)
{
  CosNotification::StructuredEvent *strEvent;
  if (event >>= strEvent)
    {
      // Any contained a structured event.
      printEvent (buffer, *strEvent, full);
    }
  else
    {
      // Any was a simple event-service style event.
      ACE_OS::strcpy (buffer, "Body: ");
      printAny (buffer, event);
    }
}

void
ChannelUtil::printPropertySeq (char *buffer, const CosNotification::PropertySeq &seq)
{
for (unsigned int i = 0; i < seq.length (); i++)
  {
    ACE_OS::strcat (buffer, "Name: <");
    ACE_OS::strcat (buffer, seq[i].name);
    ACE_OS::strcat (buffer, ">, Value: <");
    printAny (buffer, seq[i].value);
    ACE_OS::strcat (buffer, ">\n");
  }
}

void
ChannelUtil::printAny (char *buffer, const CORBA::Any &any)
{
  char val[20];

  CORBA::Short s;
  if (any >>= s)
    {
      ACE_OS::sprintf (val, "%i", s);
      ACE_OS::strcat (buffer, val);
      return;
    }

  CORBA::UShort us;
  if (any >>= us)
    {
      ACE_OS::sprintf (val, "%i", us);
      ACE_OS::strcat (buffer, val);
      return;
    }
  
  CORBA::Long l;
  if (any >>= l)
    {
      ACE_OS::sprintf (val, "%i", l);
      ACE_OS::strcat (buffer, val);
      return;
    }

  CORBA::ULong ul;
  if (any >>= ul)
    {
      ACE_OS::sprintf (val, "%i", ul);
      ACE_OS::strcat (buffer, val);
      return;
    }

  CORBA::LongLong ll;
  if (any >>= ll)
    {
      ACE_OS::sprintf (val, "%i", ll);
      ACE_OS::strcat (buffer, val);
      return;
    }
    
  CORBA::ULongLong ull;
  if (any >>= ull)
    {
      ACE_OS::sprintf (val, "%i", ull);
      ACE_OS::strcat (buffer, val);
      return;
    }

  CORBA::Float f;
  if (any >>= f)
    {
      ACE_OS::sprintf (val, "%f", f);
      ACE_OS::strcat (buffer, val);
      return;
    }

  CORBA::Double d;
  if (any >>= d)
    {
      ACE_OS::sprintf (val, "%f", d);
      ACE_OS::strcat (buffer, val);
      return;
    }

  CORBA::LongDouble ld;
  if (any >>= ld)
    {
      ACE_OS::sprintf (val, "%f", ld);
      ACE_OS::strcat (buffer, val);
      return;
    }

  CORBA::Boolean b;
  if (any >>= CORBA::Any::to_boolean (b))
    {
      if (b == 1)
        {
          ACE_OS::strcat (buffer, ACE_OS::strdup ("TRUE"));
        }
      else
        {
          ACE_OS::strcat (buffer, ACE_OS::strdup ("FALSE"));
        }
      return;
    }

  const char *str;
  if (any >>= str)
    {
      ACE_OS::strcat (buffer, str);
      return;
    }

  // Unhandled case.
  ACE_OS::strcat (buffer, ACE_OS::strdup ("[UNKNOWN_ANY]"));
  return;
}
