/*
 * Simple sequence push consumer.
 */

#include "SequencePushConsumerImpl.h"


SequencePushConsumerImpl::SequencePushConsumerImpl (CORBA::ORB_var orbVar, ChannelUtil *pUtil)
  : orbVar_ (orbVar), pUtil_ (pUtil), consumerVar_ (0)
{
  PortableServer::POA_var poaVar;
  PortableServer::ServantBase_var servantVar;
  PortableServer::ObjectId_var oidVar;
  CORBA::Object_var objVar;

  // Obtain a reference to the root POA and create the CORBA object.
  poaVar = pUtil_->getRootPoa ();
  servantVar = (PortableServer::ServantBase_var) this;
  oidVar = poaVar->activate_object (servantVar.in ());
  objVar = poaVar->id_to_reference (oidVar.in ());
  consumerVar_ = CosNotifyComm::SequencePushConsumer::_narrow (objVar.in ());

  // Connect to the channel.
  connectToChannel ();
}

SequencePushConsumerImpl::~SequencePushConsumerImpl ()
{
}

void
SequencePushConsumerImpl::disconnect_sequence_push_consumer ()
  ACE_THROW_SPEC ((CORBA::SystemException))
{
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Disconnect callback invoked\n")));
}

void
SequencePushConsumerImpl::offer_change (const CosNotification::EventTypeSeq & /*added*/, const CosNotification::EventTypeSeq & /*removed*/)
  ACE_THROW_SPEC ((CosNotifyComm::InvalidEventType,CORBA::SystemException))
{
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Offer change callback invoked\n")));
}

void
SequencePushConsumerImpl::push_structured_events (const CosNotification::EventBatch &events)
  ACE_THROW_SPEC ((CosEventComm::Disconnected,CORBA::SystemException))
{
  char message[1024];
  CORBA::ULong msgCount = events.length ();

  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Received a batch of %i message(s)\n"), msgCount));

  for (unsigned int i = 0; i < msgCount; i++)
    {
      pUtil_->printEvent (message, events[i], 0);
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Message %i of %i: %s\n"), i+1, msgCount, message));
    }
}

void
SequencePushConsumerImpl::connectToChannel ()
{
  // Get a reference to the event channel.
  CosNotifyChannelAdmin::EventChannel_var channelVar = pUtil_->getChannel ();

  // Obtain a reference to the default consumer admin.
  CosNotifyChannelAdmin::ConsumerAdmin_var adminVar;
  ACE_TRY_NEW_ENV
    {
      adminVar = channelVar->default_consumer_admin (ACE_ENV_SINGLE_ARG_PARAMETER);
      ACE_TRY_CHECK;
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Obtained default consumer admin\n")));
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to obtain default consumer admin\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to obtain default consumer admin\n")));
      exit (1);
    }
  ACE_ENDTRY;

  CosNotifyChannelAdmin::ProxyID pid;
  CosNotifyChannelAdmin::ClientType type = CosNotifyChannelAdmin::SEQUENCE_EVENT;
  CosNotifyChannelAdmin::ProxySupplier_var psVar;

  // Obtain a proxy supplier.
  ACE_TRY_NEW_ENV
    {
      psVar = adminVar->obtain_notification_push_supplier (type, pid ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      if (CORBA::is_nil (psVar.in ()))
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Obtained a null proxy supplier\n")));
          exit (1);
        }
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to obtain a proxy supplier\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to obtain a proxy supplier\n")));
      exit (1);
    }
  ACE_ENDTRY;

  // Narrow to a sequence proxy push supplier.
  CosNotifyChannelAdmin::SequenceProxyPushSupplier_var sppsVar = CosNotifyChannelAdmin::SequenceProxyPushSupplier::_narrow (psVar.in ());
  if (CORBA::is_nil (sppsVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Narrowed to a null sequence proxy push supplier\n")));
      exit (1);
    }

  // Set up custom MaxBatchSize and PacingInterval values.  Comment out this code to inherit the
  // values from the consumer admin.
  CosNotification::QoSProperties qos = CosNotification::QoSProperties (2);
  qos.length (2);
  TimeBase::TimeT pacing = 10L * 10000000L;
  qos[0].name = ACE_OS::strdup (ACE_TEXT ("PacingInterval"));
  qos[0].value <<= pacing;
  qos[1].name = ACE_OS::strdup (ACE_TEXT ("MaximumBatchSize"));
  qos[1].value <<= (int) 10;
  ACE_TRY_NEW_ENV
    {
      sppsVar->set_qos (qos ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to set QoS properties on the sequence proxy push supplier.\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("ERROR: Failed to set QoS properties on the sequence proxy push supplier.\n")));
      exit (1);
    }
  ACE_ENDTRY;

  // Connect.
  ACE_TRY_NEW_ENV
    {
      sppsVar->connect_sequence_push_consumer (consumerVar_ ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Connected consumer to proxy\n")));
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to connect consumer to proxy\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to connect consumer to proxy\n")));
      exit (1);
    }
  ACE_ENDTRY;
}

int
main (int argc, char **argv)
{
  CORBA::ORB_var orbVar;
  ChannelUtil *pUtil;
  SequencePushConsumerImpl *pConsumerImpl;

  // Initialise the ORB reference.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Creating ORB reference\n")));
  orbVar = CORBA::ORB_init (argc, argv);
  if (CORBA::is_nil (orbVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Could not create the ORB\n")));
      exit (1);
    }

  // Obtain a reference to the channel utility.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Creating channel utility\n")));
  pUtil = new ChannelUtil (orbVar);

  // Create the consumer object.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Creating consumer\n")));
  pConsumerImpl = new SequencePushConsumerImpl (orbVar, pUtil);

  // Enter the orb run loop.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Entering ORB run loop\n")));
  orbVar->run ();

  // Clean up and exit.
  delete pConsumerImpl;
  delete pUtil;
  return 0;
}
