/*
 * Simple structured push supplier.
 */

#include "StructuredPushSupplierImpl.h"


StructuredPushSupplierImpl::StructuredPushSupplierImpl (CORBA::ORB_var orbVar, ChannelUtil *pUtil)
  : orbVar_ (orbVar), pUtil_ (pUtil), supplierVar_ (0), proxyVar_ (0)
{
  PortableServer::POA_var poaVar;
  PortableServer::ServantBase_var servantVar;
  PortableServer::ObjectId_var oidVar;
  CORBA::Object_var objVar;

  // Obtain a reference to the root POA and create the CORBA object.
  poaVar = pUtil_->getRootPoa ();
  servantVar = (PortableServer::ServantBase_var) this;
  oidVar = poaVar->activate_object (servantVar.in ());
  objVar = poaVar->id_to_reference (oidVar);
  supplierVar_ = CosNotifyComm::StructuredPushSupplier::_narrow (objVar.in ());

  // Connect to the channel.
  connectToChannel ();
}

StructuredPushSupplierImpl::~StructuredPushSupplierImpl ()
{
}

void
StructuredPushSupplierImpl::disconnect_structured_push_supplier ()
  ACE_THROW_SPEC ((CORBA::SystemException))
{
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Disconnect callback invoked\n")));
}

void
StructuredPushSupplierImpl::subscription_change (const CosNotification::EventTypeSeq & /*added*/, const CosNotification::EventTypeSeq & /*removed*/)
  ACE_THROW_SPEC ((CosNotifyComm::InvalidEventType,CORBA::SystemException))
{
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Subscription change callback invoked\n")));
}

void
StructuredPushSupplierImpl::pushEvents ()
{
  CosNotification::StructuredEvent_var eventVar;
  int sent = 0;
  char message[256];
  ACE_Time_Value atv;
  long delay = 2000L;

  atv.msec (delay);

  while (1)
    {
      eventVar = pUtil_->createStructuredEvent (sent++);
      pUtil_->printEvent (message, eventVar.in (), 0);
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Sending event: %s\n"), message));
      proxyVar_->push_structured_event (eventVar.in ());
      ACE_OS::sleep (atv);
    }
}

void
StructuredPushSupplierImpl::connectToChannel ()
{
  // Get a reference to the event channel.
  CosNotifyChannelAdmin::EventChannel_var channelVar = pUtil_->getChannel ();

  // Obtain a reference to the default supplier admin.
  CosNotifyChannelAdmin::SupplierAdmin_var adminVar;
  ACE_TRY_NEW_ENV
    {
      adminVar = channelVar->default_supplier_admin (ACE_ENV_SINGLE_ARG_PARAMETER);
      ACE_TRY_CHECK;
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Obtained default supplier admin\n")));
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to obtain default supplier admin\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to obtain default supplier admin\n")));
      exit (1);
    }
  ACE_ENDTRY;

  CosNotifyChannelAdmin::ProxyID pid;
  CosNotifyChannelAdmin::ClientType type = CosNotifyChannelAdmin::STRUCTURED_EVENT;
  CosNotifyChannelAdmin::ProxyConsumer_var pcVar;

  // Obtain a proxy consumer.
  ACE_TRY_NEW_ENV
    {
      pcVar = adminVar->obtain_notification_push_consumer (type, pid ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      if (CORBA::is_nil (pcVar.in ()))
        {
          ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Obtained a null proxy consumer\n")));
          exit (1);
        }
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to obtain a proxy consumer\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to obtain a proxy consumer\n")));
      exit (1);
    }
  ACE_ENDTRY;

  // Narrow to a structured proxy push consumer.
  proxyVar_ = CosNotifyChannelAdmin::StructuredProxyPushConsumer::_narrow (pcVar.in ());
  if (CORBA::is_nil (proxyVar_.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Narrowed to a null structured proxy push consumer\n")));
      exit (1);
    }

  // Connect.
  ACE_TRY_NEW_ENV
    {
      proxyVar_->connect_structured_push_supplier (supplierVar_ ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("Connected supplier to proxy\n")));
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to connect supplier to proxy\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to connect supplier to proxy\n")));
      exit (1);
    }
  ACE_ENDTRY;
}

int
main (int argc, char **argv)
{
  CORBA::ORB_var orbVar;
  ChannelUtil *pUtil;
  StructuredPushSupplierImpl *pSupplierImpl;

  // Initialise the ORB reference.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Creating ORB reference\n")));
  orbVar = CORBA::ORB_init (argc, argv);
  if (CORBA::is_nil (orbVar.in ()))
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Could not create the ORB\n")));
      exit (1);
    }

  // Obtain a reference to the channel utility.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Creating channel utility\n")));
  pUtil = new ChannelUtil (orbVar);

  // Create the supplier object.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Creating supplier\n")));
  pSupplierImpl = new StructuredPushSupplierImpl (orbVar, pUtil);

  // Send events.
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Sending events\n")));
  pSupplierImpl->pushEvents ();

  // Clean up and exit.
  delete pSupplierImpl;
  delete pUtil;
  return 0;
}
