/*
 * This CORBA server delegates to a (singleton) VideoServiceProvider object
 * that manages VideoSupplier objects using the Trading Service. The program
 * registers a Service Type describing a VideoSupplier offer. Each
 * VideoSupplier object exports its offer to the service.
 */

#include "Trading/video/VideoCompany.h"
#include "Trading/video/VideoServiceProvider.h"
#include "Trading/video/TradingVideoSupplier.h"
#include "Trading/video/VideoSupplier.h"

#include "Common/ObjectAdapter.h"

#include <tao/corba.h>
#include <tao/AnyTypeCode/TypeCode.h>
#include <tao/PortableServer/PortableServer.h>
#include <ace/OS.h>


int main (int argc, char **argv)
{
  CORBA::ORB_var orbVar;
  CORBA::Object_var objVar;
  CosTrading::Lookup_var lookupVar;
  CosTradingRepos::ServiceTypeRepository_var reposVar;

  // Initialise the ORB.
  ACE_TRY_NEW_ENV
    {
      orbVar = CORBA::ORB_init (argc, argv ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      ACE_DEBUG ((LM_INFO, ACE_TEXT ("INFO: Initialised ORB\n")));
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to initialise ORB\n"));
      exit (-1);
    }
  ACE_ENDTRY;

  // Get a reference to the trading service.
  ACE_TRY_NEW_ENV
    {
      objVar = orbVar->resolve_initial_references (ACE_TEXT ("TradingService") ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
      lookupVar = CosTrading::Lookup::_narrow (objVar.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to resolve/narrow to the trading service\n"));
      exit (-1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to resolve/narrow to the trading service\n")));
      exit (-1);
    }
  ACE_ENDTRY;

  // Get a reference to the service type repository.
  ACE_TRY_NEW_ENV
    {
      objVar = lookupVar->type_repos (ACE_ENV_SINGLE_ARG_PARAMETER);
      ACE_TRY_CHECK;
      reposVar = CosTradingRepos::ServiceTypeRepository::_narrow (objVar.in () ACE_ENV_ARG_PARAMETER);
      ACE_TRY_CHECK;
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to obtain reference to the service type repository\n"));
      exit (-1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to obtain reference to the service type repository\n")));
      exit (-1);
    }
  ACE_ENDTRY;

  // Create the service provider.
  VideoSupplier::pServiceProvider_ = new VideoServiceProvider (orbVar);

  // Register the service type.

  CosTradingRepos::ServiceTypeRepository::PropStructSeq properties;
  properties.length (3);

  properties[0].name = ACE_OS::strdup (ACE_TEXT ("type"));
  properties[0].value_type = CORBA::_tc_string;
  properties[0].mode = CosTradingRepos::ServiceTypeRepository::PROP_MANDATORY_READONLY;

  properties[1].name = ACE_OS::strdup (ACE_TEXT ("title"));
  properties[1].value_type = CORBA::_tc_string;
  properties[1].mode = CosTradingRepos::ServiceTypeRepository::PROP_MANDATORY_READONLY;

  properties[2].name = ACE_OS::strdup (ACE_TEXT ("cost"));
  properties[2].value_type = CORBA::_tc_float;
  properties[2].mode = CosTradingRepos::ServiceTypeRepository::PROP_MANDATORY;

  CosTradingRepos::ServiceTypeRepository::ServiceTypeNameSeq defaultSupers;

  ACE_TRY_NEW_ENV
    {
      reposVar->add_type (ACE_TEXT ("IDL:video_supplier:1.0"),
                          ACE_TEXT ("IDL:video_supplier_if:1.0"),
                          properties,
                          defaultSupers
                          ACE_ENV_ARG_PARAMETER);
    }
  ACE_CATCH (CosTradingRepos::ServiceTypeRepository::ServiceTypeExists, ste)
    {
      ACE_PRINT_EXCEPTION (ste, ACE_TEXT ("ERROR: Failed to add the service type\n"));
      exit (1);
    }
  ACE_CATCH (CosTradingRepos::ServiceTypeRepository::InterfaceTypeMismatch, itm)
    {
      ACE_PRINT_EXCEPTION (itm, ACE_TEXT ("ERROR: Failed to add the service type\n"));
      exit (1);
    }
  ACE_CATCH (CosTrading::DuplicatePropertyName, dpn)
    {
      ACE_PRINT_EXCEPTION (dpn, ACE_TEXT ("ERROR: Failed to add the service type\n"));
      exit (1);
    }
  ACE_CATCH (CosTradingRepos::ServiceTypeRepository::ValueTypeRedefinition, vtr)
    {
      ACE_PRINT_EXCEPTION (vtr, ACE_TEXT ("ERROR: Failed to add the service type\n"));
      exit (1);
    }
  ACE_CATCH (CosTrading::UnknownServiceType, ust)
    {
      ACE_PRINT_EXCEPTION (ust, ACE_TEXT ("ERROR: Failed to add the service type\n"));
      exit (1);
    }
  ACE_CATCHANY
    {
      ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION, ACE_TEXT ("ERROR: Failed to add the service type\n"));
      exit (1);
    }
  ACE_CATCHALL
    {
      ACE_DEBUG ((LM_ERROR, ACE_TEXT ("ERROR: Failed to add the service type\n")));
      exit (1);
    }
  ACE_ENDTRY;

  // Create a few arbitrary categories.
  const char *cats[] = {ACE_TEXT ("Films"),           //0
                        ACE_TEXT ("Sporting Events"), //1
                        ACE_TEXT ("News Broadcasts"), //2
                        ACE_TEXT ("Soap Operas")      //3
                       };

  // Create a few arbitrary videos.
  const char *vids[] = {ACE_TEXT ("Bridge On The River Kwai"), //0
                        ACE_TEXT ("Mutiny On The Bounty"),     //1
                        ACE_TEXT ("Lawrence Of Arabia"),       //2
                        ACE_TEXT ("Star Wars"),                //3
                        ACE_TEXT ("Great Expectations"),       //4
                        ACE_TEXT ("FA Cup Final"),             //5
                        ACE_TEXT ("BBC"),                      //6
                        ACE_TEXT ("CNN"),                      //7
                        ACE_TEXT ("Coronation Street"),        //8
                        ACE_TEXT ("Neighbours")                //9
                       };

  // Create a few arbitary video suppliers and add their offers into the trading service.

  TradingVideoSupplier videosRUs (ACE_TEXT ("Videos R us"));
  videosRUs.addCategory (cats[0]);
  videosRUs.addCategory (cats[1]);
  videosRUs.addVideo (cats[0], vids[0]);
  videosRUs.addVideo (cats[0], vids[1]);
  videosRUs.addVideo (cats[1], vids[5]);
  videosRUs.exportOffer (lookupVar.in ());

  TradingVideoSupplier videos4Sale (ACE_TEXT ("Videos for Sale"));
  videos4Sale.addCategory (cats[0]);
  videos4Sale.addCategory (cats[2]);
  videos4Sale.addVideo (cats[0], vids[2]);
  videos4Sale.addVideo (cats[0], vids[3]);
  videos4Sale.addVideo (cats[0], vids[4]);
  videos4Sale.addVideo (cats[2], vids[6]);
  videos4Sale.addVideo (cats[2], vids[7]);
  videos4Sale.exportOffer (lookupVar.in ());

  TradingVideoSupplier videosOnDemand (ACE_TEXT ("Videos on Demand"));
  videosOnDemand.addCategory (cats[2]);
  videosOnDemand.addCategory (cats[3]);
  videosOnDemand.addVideo (cats[2], vids[6]);
  videosOnDemand.addVideo (cats[3], vids[8]);
  videosOnDemand.addVideo (cats[3], vids[9]);
  videosOnDemand.exportOffer (lookupVar.in ());

  // Accept incoming requests on this server.
  VideoSupplier::pServiceProvider_->activateManager ();
  ACE_DEBUG ((LM_INFO, ACE_TEXT ("Ready...\n")));
  orbVar->run ();

  // Clean up.
  delete VideoSupplier::pServiceProvider_;
  return 0;
}
